@@ -215,7 +215,10 @@ and this function returns the response data directly."
   "Function intended for use as an entry of `ansi-osc-handlers'.
 TEXT is a still unparsed message received from the interpreter."
   (drepl--log-message "read %s" text)
-  (drepl--handle-response drepl--current (drepl--json-decode text)))
+  (condition-case err
+      (drepl--handle-response drepl--current (drepl--json-decode text))
+    (t (drepl--log-message "error: %s" err)
+       (message "Error processing dREPL message"))))
 
 (defun drepl--handle-response (repl data)
   "React to message DATA coming from the REPL process."
@@ -400,13 +403,13 @@ Otherwise, make an eval request."
         (repl (with-current-buffer
                   (if proc (process-buffer proc) (current-buffer))
                 (drepl--get-repl nil t))))
-    (if (not (eq (drepl--status repl) 'rawio))
+    (if (eq (drepl--status repl) 'rawio)
         (progn
-          (when-let ((hist (drepl--history-variable repl)))
-            (add-to-history hist string comint-input-ring-size))
-            (drepl--eval repl string))
-      (drepl--log-message "send raw %s" string)
-      (comint-simple-send proc string))))
+          (drepl--log-message "send raw %s" string)
+          (comint-simple-send proc string))
+      (when-let ((hist (drepl--history-variable repl)))
+        (add-to-history hist string comint-input-ring-size))
+      (drepl--eval repl string))))
 
 (defun drepl-eval (code)
   "Evaluate CODE string in the current buffer's REPL."
@@ -501,25 +504,18 @@ See that variable's docstring for a description of CALLBACK."
 
 ;;; REPL restart
 
-(cl-defgeneric drepl--restart (repl hard)
-  "Generic method to restart a REPL.
-HARD should be as described in `drepl-restart', but it not used
-in the default implementation."
-  (ignore hard)
+(cl-defgeneric drepl--restart (repl)
+  "Generic method to restart a REPL."
   (with-current-buffer (drepl--buffer repl)
     (when-let ((proc (drepl--process repl)))
       (kill-process proc)
       (while (accept-process-output proc)))
     (drepl--get-buffer-create (type-of repl) nil)))
 
-(defun drepl-restart (&optional hard)
-  "Restart the current REPL.
-Some REPLs by default perform a soft reset by deleting all user
-variables without killing the interpreter.  In those cases, a
-prefix argument or non-nil HARD argument can be used to force a
-hard reset."
-  (interactive "P")
-  (drepl--restart (drepl--get-repl nil t) hard))
+(defun drepl-restart ()
+  "Restart the current REPL."
+  (interactive)
+  (drepl--restart (drepl--get-repl nil t)))
 
 ;;; REPL initialization
 
